from enum import Enum
from typing import Tuple, List
       
class UserSpace:
    """
    Define the space where user code will be executed. 
    This is encapsulated in a python module so all the imports and variables are separated out from the rest.
    The code is executed on a kernel such as BaseKernel or IPythonKernel
    """
    def __init__(self, df_types: list):
        """
        Args:
            df_types (list): a list of interested data frames types which can be either cycai.DataFrame or pandas.DataFrame
        """
        self.df_types = df_types

    def globals(self):
        raise NotImplementedError('Abstract method not implemented.')

    @classmethod
    def fullname(cls, klass):
        module = klass.__module__
        if module == 'builtins':
            return klass.__qualname__
        return module + '.' + klass.__qualname__

    def get_current_dfs(self) -> List[Tuple[str, str, str]]:
        """
        Get a list of data frame with type in `df_types` from `user_space`
        
        Returns:
            list: list of data frame with the following information (name, id, type)
        """
        user_space = self.globals()
        names = list(user_space)
        df_list: List[Tuple[str, str, str]] = []
        
        for name in names:
            df_type = type(user_space[name])
            if df_type in self.df_types:
                df_list.append(
                    (name, id(user_space[name]), self.fullname(df_type)))

        return df_list
